# Program to calculate docking root mean square deviation (RMSD) between 
# the atomic coordinates of two PDB files for non-hydrogen atoms. 
# Both files should have exactly the same number of atoms.
#
# Author: Dr. Walter Filgueira de Azevedo Jr.
# Date: January, 31st 2016.

def read_PDB_return_coord(pdb_in):
    """Function to read atomic coordinates from a PDB file"""
    
    # Import libraries
    import sys
    import numpy as np
    
    # Setup empty lists
    x = []
    y = []
    z = []
    
    # Try to open PDB file
    try:
        my_PDB_fo = open(pdb_in,"r")
    except IOError:
        sys.exit("\nI can't file "+pdb_in+" file!")
    
    # Looping through PDB file
    for line in my_PDB_fo:
        if line[0:6] == "HETATM" or line[0:6] == "ATOM  ":
            if line[13:14] != "H":
                x.append(float(line[30:38]))
                y.append(float(line[38:46]))
                z.append(float(line[46:54]))
    
    # Convert list to array
    x_coord = np.array(x)
    y_coord = np.array(y)
    z_coord = np.array(z)
    
    # Return arrays
    return x_coord, y_coord, z_coord

def calc_rmsd(x1,y1,z1,x2,y2,z2):
    """Function to calculate RMSD"""
    
    # Import library
    import math
    
    # Set count to zero
    sum_d2 = 0
    
    # Assign len(x1) to n (size of array)
    n =  len(x1)
    
    # Looping through all atomic coordinates to calculate sum of d2
    for i in range(n):
        # Calculate the square of Euclidian distance between two points
        d2 =  ( x1[i] - x2[i] )**2 + (y1[i] - y2[i])**2 + (z1[i] - z2[i])**2  
        sum_d2 += d2
    
    # If n> 0 calculate rmsd
    if n > 0 :
        rmsd = math.sqrt(sum_d2/n)
    else:
        rmsd =  None
    
    # Return rmsd
    return rmsd
    
def main():
    
    # Read file names
    my_file1 = input("\nType first PDB file name  => ")
    my_file2 = input("\nType second PDB file name => ")
    
    # Call read_PDB_return_coord() to get atomic coordinates as arrays
    x1,y1,z1 = read_PDB_return_coord(my_file1)
    
    # Call read_PDB_return_coord() to get atomic coordinates as arrays
    x2,y2,z2 = read_PDB_return_coord(my_file2)
    
    # Call calc_rmsd()
    rmsd = calc_rmsd(x1,y1,z1,x2,y2,z2)
    
    # Get the size of the array
    n = len(x1)
    
    # Show results
    print("\nDocking RMSD = %6.3f"%rmsd," A for %5d"%n," atoms.")

main()
